/*
 * Image canonicalisation function for use by callers feeding
 *  images into structure.c's routines.
 */
/*
 *  This is part of ypp-sc-tools, a set of third-party tools for assisting
 *  players of Yohoho Puzzle Pirates.
 * 
 *  Copyright (C) 2009 Ian Jackson <ijackson@chiark.greenend.org.uk>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *  Yohoho and Puzzle Pirates are probably trademarks of Three Rings and
 *  are used without permission.  This program is not endorsed or
 *  sponsored by Three Rings.
 */

#ifndef STRUCTURE_H
#define STRUCTURE_H


#include "convert.h"


typedef struct {
  Rgb rgb; /* on screen */
  char c; /* canonical */
} CanonColourInfo;

extern const CanonColourInfo canoncolourinfos[];

CanonImage *alloc_canon_image(int w, int h);

#define CANONICALISE_IMAGE(im,w,h, COMPUTE_RGB) do{		\
    /* compute_rgb should be a number of statements, or		\
     * a block, which assigns to				\
     *   Rgb rgb;						\
     * given the values of					\
     *   int x,y;						\
     * all of which are anamorphic.  Result is stored in im.	\
     * The COMPUTE_RGB is executed exactly once for		\
     * each pixel in reading order.				\
     */								\
    (im)= alloc_canon_image((w), (h));				\
    (im)->rgb= alloc_rgb_image((w), (h));			\
								\
    int x,y;							\
    unsigned long last_rgb= ~0UL;				\
    int last_c= -1;						\
    for (y=0; y<(h); y++) {					\
      for (x=0; x<(w); x++) {					\
        const CanonColourInfo *cci;				\
        unsigned long rgb;					\
	COMPUTE_RGB;						\
        CANONIMG_ALSO_STORERGB((im)->rgb);			\
	if (rgb == last_rgb) {					\
	  (im)->d[y*(w) + x]= last_c;				\
	} else {						\
	  for (cci=canoncolourinfos; cci->c; cci++) {		\
	    if (cci->rgb == rgb) {				\
	      last_rgb= rgb;					\
	      (im)->d[y*(w) + x]= last_c= cci->c;		\
	      break;						\
	    }							\
	  }							\
	}							\
      }								\
      if (DEBUGP(rect)) {					\
	fprintf(debug, "%4d ",y);				\
	fwrite(im->d + y*(w), 1,(w), debug);			\
	fputc('\n',debug);					\
      }								\
    }								\
    debug_flush();						\
  }while(0)


#define CANONIMG_ALSO_STORERGB(ri)		\
  do{						\
    unsigned char *rip= RI_PIXEL((ri),x,y);	\
    rip[0]= rgb >> 16;				\
    rip[1]= rgb >> 8;				\
    rip[2]= rgb;				\
  }while(0)


#endif /*STRUCTURE_H*/
